/* BlinkenLib Copyright 2004-2011 Stefan Schuermans <stefan@blinkenarea.org>
   Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html a
   blinkenarea.org project */

#ifndef INC_BlinkenLib_BlinkenFrame
#define INC_BlinkenLib_BlinkenFrame

typedef struct sBlinkenFrame stBlinkenFrame;

typedef enum eBlinkenProto { BlinkenProtoNone, BlinkenProtoBlp,
  BlinkenProtoEblp, BlinkenProtoMcuf
} etBlinkenProto;

stBlinkenFrame *BlinkenFrameNew(int height, int width, int channels,
                                int maxval, int duration);

stBlinkenFrame *BlinkenFrameClone(stBlinkenFrame * pSrcFrame);

void BlinkenFrameFree(stBlinkenFrame * pFrame);

void BlinkenFrameClear(stBlinkenFrame * pFrame);

int BlinkenFrameGetHeight(stBlinkenFrame * pFrame);
int BlinkenFrameGetWidth(stBlinkenFrame * pFrame);
int BlinkenFrameGetChannels(stBlinkenFrame * pFrame);
int BlinkenFrameGetMaxval(stBlinkenFrame * pFrame);

int BlinkenFrameGetDuration(stBlinkenFrame * pFrame);
void BlinkenFrameSetDuration(stBlinkenFrame * pFrame, int duration);

unsigned char BlinkenFrameGetPixel(stBlinkenFrame * pFrame, int y, int x,
                                   int c);
void BlinkenFrameSetPixel(stBlinkenFrame * pFrame, int y, int x, int c,
                          unsigned char val);

unsigned long BlinkenFrameGetColor(stBlinkenFrame * pFrame, int y, int x);
void BlinkenFrameSetColor(stBlinkenFrame * pFrame, int y, int x,
                          unsigned long color);

int BlinkenFrameIsEmpty(stBlinkenFrame * pFrame);
// returns 1 if frame is empty (i.e. black), returns 0 otherwise

int BlinkenFrameCompare(stBlinkenFrame * pFrame1, stBlinkenFrame * pFrame2);
// returns -1 for frame1 smaller, 0 for equal, 1 for frame2 smaller

void BlinkenFrameResize(stBlinkenFrame * pFrame, int height, int width,
                        int channels, int maxval);
void BlinkenFrameScale(stBlinkenFrame * pFrame, int height, int width);
void BlinkenFrameColorize(stBlinkenFrame * pFrame, int channels, int mode,
                          int step);

char *BlinkenFrameToString(stBlinkenFrame * pFrame);

int BlinkenFrameToNetwork(stBlinkenFrame * pFrame, etBlinkenProto proto,
                          char *pData, int maxLength);
// returns length or -1 on error

stBlinkenFrame *BlinkenFrameFromNetwork(char *pData, int length,
                                        etBlinkenProto * pProto);
// returns protocol in *pProto if pProto not NULL

#endif // #ifndef INC_BlinkenLib_BlinkenFrame
